from typing import List

from pycode2seq.inference.common.node import Node
from pycode2seq.inference.methods.model import ParameterNode
from pycode2seq.inference.methods.splitter import Splitter
from pycode2seq.inference.parsing.utils import decompress_type_label


class KotlinSplitter(Splitter):

    def _method_name(self) -> str:
        return "functionDeclaration"

    def _method_return_type_node(self) -> str:
        return "type"

    def _method_name_node(self) -> str:
        return "simpleIdentifier"

    def _class_declaration_node(self) -> List[str]:
        return ["classDeclaration", "objectDeclaration"]

    def _class_name_node(self) -> str:
        return "simpleIdentifier"

    def _method_parameter_node(self) -> str:
        return "functionValueParameters"

    def _method_single_parameter_node(self) -> List[str]:
        return ["functionValueParameter"]

    def _parameter_return_type_node(self) -> str:
        return "type"

    def _parameter_name_node(self) -> str:
        return "simpleIdentifier"

    def _get_list_of_parameters(self, root: Node) -> List[ParameterNode]:
        if root is None:
            return []
        return [
            self._get_parameter_info_from_node(child) for child in root.children
            if decompress_type_label(child.type_label)[0] == self._method_single_parameter_node()[0]
        ]



from dataclasses import dataclass
from typing import List
from pycode2seq.inference.common.node import Node


@dataclass
class PathPiece:
    nodes: List[Node]
    terminal: Node

    def __str__(self) -> str:
        return "->".join(node.type_label for node in self.nodes) + f"->({self.terminal.get_token()})"

    def __len__(self) -> int:
        return len(self.nodes)


@dataclass
class ASTPath:
    nodes: List[Node]
    left_terminal: Node
    right_terminal: Node

    def __str__(self) -> str:
        return f"{self.left_terminal.get_normalized_token()}," + "|".join(
            node.type_label for node in self.nodes) + f",{self.right_terminal.get_normalized_token()}"


@dataclass
class ExtractingParams:
    max_length: int
    max_width: int
    paths_per_method: int


def extract_paths(root: Node, params: ExtractingParams) -> List[ASTPath]:
    paths = []
    extract_pieces(root, paths, params)
    return paths


def collect_paths(pieces: List[List[PathPiece]], node: Node, params: ExtractingParams) -> List[ASTPath]:
    paths = []
    for i, left_pieces in enumerate(pieces):
        for right_pieces in pieces[i + 1: i + 1 + params.max_width]:
            for left_piece in left_pieces:
                for right_piece in right_pieces:
                    left_token = left_piece.terminal.get_normalized_token()
                    right_token = right_piece.terminal.get_normalized_token()
                    if len(left_piece) + len(right_piece) + 1 <= params.max_length and left_token and right_token:
                        path = left_piece.nodes + [node] + list(reversed(right_piece.nodes))
                        paths.append(ASTPath(path, left_piece.terminal, right_piece.terminal))

    return paths


def extract_pieces(root: Node, paths: List[ASTPath], params: ExtractingParams) -> List[List[PathPiece]]:
    pieces = []
    for child in root.children:
        if child.is_leaf():
            pieces.append([PathPiece([], child)])
        else:
            pieces.append(
                [PathPiece(piece.nodes + [child], piece.terminal)
                 for child_pieces in extract_pieces(child, paths, params)
                 for piece in child_pieces if len(piece) + 1 <= params.max_length])

    paths.extend(collect_paths(pieces, root, params))

    return pieces



from typing import List, Optional
from antlr4.tree.Tree import TerminalNode
from pycode2seq.inference.common.node import Node
from antlr4 import ParserRuleContext
from antlr4 import Lexer


def convert_rule_context(cntx: ParserRuleContext, rule_names: List[str], parent: Optional[Node], lexer: Lexer) -> Node:
    type_label = rule_names[cntx.getRuleIndex()]
    current_node = Node(type_label, parent, None)
    children = []
    for child in cntx.getChildren():
        if isinstance(child, TerminalNode):
            children.append(convert_terminal(child, current_node, lexer))
        else:
            children.append(convert_rule_context(child, rule_names, current_node, lexer))

    current_node.set_children(children)

    return current_node


def convert_terminal(terminal_node: TerminalNode, parent: Node, lexer: Lexer):
    return Node(lexer.symbolicNames[terminal_node.getSymbol().type], parent, terminal_node.getSymbol().text)


def compress_tree(root: Node) -> Node:
    if len(root.children) == 1:
        child = compress_tree(root.children[0])
        compressed_node = Node(
            root.type_label + "|" + child.type_label,
            root.parent,
            child.get_token()
        )
        compressed_node.set_children(child.children)
        return compressed_node

    root.set_children([compress_tree(child) for child in root.children])
    return root


def decompress_type_label(type_label: str) -> List[str]:
    return type_label.split("|")


def shorten_nodes(root: Node) -> Node:
    parts = decompress_type_label(root.type_label)
    label = parts[0]

    if len(parts) > 1:
        label += "/" + parts[-1]

    children = [shorten_nodes(child) for child in root.children]

    new_node = Node(label, root.parent, root.token)

    new_node.set_children(children)

    return new_node



from argparse import ArgumentParser
from typing import Dict
from code2seq.utils.vocabulary import Vocabulary


def combine_dicts(dict1: Dict[str, int], dict2: Dict[str, int]):
    new_items = set(key for key in dict2.keys()).difference(key for key in dict1.keys())
    new_id = max(item for _, item in dict1.items())

    for item in new_items:
        dict1[item] = new_id
        new_id += 1

    return dict1


if __name__ == "__main__":
    arg_parser = ArgumentParser()
    arg_parser.add_argument("old_voc", type=str)
    arg_parser.add_argument("second_voc", type=str)
    arg_parser.add_argument("output", type=str)
    args = arg_parser.parse_args()

    old_voc = Vocabulary.load_vocabulary(args.old_voc)
    second_voc = Vocabulary.load_vocabulary(args.second_voc)

    new_voc = Vocabulary(
        old_voc.token_to_id,
        combine_dicts(old_voc.node_to_id, second_voc.node_to_id),
        second_voc.label_to_id,
    )

    new_voc.dump_vocabulary(args.output)



from argparse import ArgumentParser
from os import path
from random import seed

from code2seq.preprocessing.astminer_to_code2seq import preprocess_csv

if __name__ == "__main__":
    arg_parser = ArgumentParser()
    arg_parser.add_argument("data", type=str)
    arg_parser.add_argument("--shuffle", action="store_true")
    args = arg_parser.parse_args()
    data_path = path.join("data", args.data)

    seed(7)
    for holdout in ["train", "val", "test"]:
        print(f"preprocessing {holdout} data")
        preprocess_csv("data", args.data, holdout, args.shuffle)



from typing import List

from pycode2seq.inference.common.node import Node
from pycode2seq.inference.methods.model import ParameterNode
from pycode2seq.inference.methods.splitter import Splitter
from pycode2seq.inference.parsing.utils import decompress_type_label


class JavaSplitter(Splitter):

    def _method_name(self) -> str:
        return "methodDeclaration"

    def _method_return_type_node(self) -> str:
        return "typeTypeOrVoid"

    def _method_name_node(self) -> str:
        return "IDENTIFIER"

    def _class_declaration_node(self) -> List[str]:
        return ["classDeclaration"]

    def _class_name_node(self) -> str:
        return "IDENTIFIER"

    def _method_parameter_node(self) -> str:
        return "formalParameters"

    def _method_single_parameter_node(self) -> List[str]:
        return ["formalParameter", "lastFormalParameter"]

    def _parameter_return_type_node(self) -> str:
        return "typeType"

    def _parameter_name_node(self) -> str:
        return "variableDeclaratorId"

    def _get_list_of_parameters(self, root: Node) -> List[ParameterNode]:
        if root is None:
            return []

        inner_parameters_root = root.get_child_of_type("formalParameterList")
        params_root = root if inner_parameters_root is None else inner_parameters_root

        if decompress_type_label(params_root.type_label)[-1] == self._method_single_parameter_node():
            return [self._get_parameter_info_from_node(params_root)]

        return [
            self._get_parameter_info_from_node(child) for child in params_root.children
            if decompress_type_label(child.type_label)[0] in self._method_single_parameter_node()
        ]



from abc import ABC, abstractmethod
from typing import List, Optional

from pycode2seq.inference.common.utils import pre_order
from pycode2seq.inference.common.node import Node
from pycode2seq.inference.methods.model import ElementNode, MethodInfo, MethodNode, ParameterNode
from pycode2seq.inference.parsing.utils import decompress_type_label


class Splitter(ABC):
    @abstractmethod
    def _method_name(self) -> str:
        raise NotImplementedError

    @abstractmethod
    def _method_return_type_node(self) -> str:
        raise NotImplementedError

    @abstractmethod
    def _method_name_node(self) -> str:
        raise NotImplementedError

    @abstractmethod
    def _class_declaration_node(self) -> List[str]:
        raise NotImplementedError

    @abstractmethod
    def _class_name_node(self) -> str:
        raise NotImplementedError

    @abstractmethod
    def _method_parameter_node(self) -> str:
        raise NotImplementedError

    @abstractmethod
    def _method_single_parameter_node(self) -> List[str]:
        raise NotImplementedError

    @abstractmethod
    def _parameter_return_type_node(self) -> str:
        raise NotImplementedError

    @abstractmethod
    def _parameter_name_node(self) -> str:
        raise NotImplementedError

    @abstractmethod
    def _get_list_of_parameters(self, root: Node) -> List[ParameterNode]:
        raise NotImplementedError

    def split_on_methods(self, root: Node) -> List[MethodInfo]:
        method_roots = [node for node in pre_order(root) if
                        decompress_type_label(node.type_label)[-1] == self._method_name()]
        return [self._collect_method_info(root) for root in method_roots]

    def _collect_method_info(self, method_node: Node) -> MethodInfo:
        method_name = method_node.get_child_of_type(self._method_name_node())
        method_return_type_node = method_node.get_child_of_type(self._method_return_type_node())
        if method_return_type_node:
            method_return_type_node.token = self._collect_parameter_token(method_return_type_node)

        class_root = self._get_enclosing_class(method_node)
        class_name = class_root.get_child_of_type(self._class_name_node()) if class_root is not None else None

        parameters_root = method_node.get_child_of_type(self._method_parameter_node())
        parameters_list = self._get_list_of_parameters(parameters_root)

        return MethodInfo(
            MethodNode(method_node, method_return_type_node, method_name),
            ElementNode(class_root, class_name),
            parameters_list
        )

    def _get_enclosing_class(self, root: Node) -> Optional[Node]:
        if decompress_type_label(root.type_label)[-1] in self._class_declaration_node():
            return root

        parent = root.parent
        if parent is not None:
            return self._get_enclosing_class(parent)

        return None

    def _get_parameter_info_from_node(self, root: Node) -> ParameterNode:
        return_type_node = root.get_child_of_type(self._parameter_return_type_node())
        return_type_node.token = self._collect_parameter_token(return_type_node)

        return ParameterNode(
            root,
            return_type_node,
            root.get_child_of_type(self._parameter_name_node())
        )

    def _collect_parameter_token(self, root: Node) -> str:
        if root.is_leaf():
            return root.token

        return "".join([self._collect_parameter_token(child) for child in root.children])



from argparse import ArgumentParser
from os.path import join

import torch

from omegaconf import OmegaConf
from code2seq.model import Code2Seq
from code2seq.utils.vocabulary import Vocabulary


weights_to_expand = [
    "encoder.node_embedding.weight",
]


def expand_weights(checkpoint_path, config_path, output_path):
    device = torch.device("cuda" if torch.cuda.is_available() else "cpu")

    net = torch.load(checkpoint_path)

    config = OmegaConf.load(config_path)
    vocabulary_path = join(config.data_folder, config.dataset.name, config.vocabulary_name)
    vocabulary = Vocabulary.load_vocabulary(vocabulary_path)
    model = Code2Seq(config, vocabulary)

    deltas = {}

    for key in weights_to_expand:
        weight1 = net["state_dict"][key]
        weight2 = model.state_dict()[key]

        delta = weight2.shape[0] - weight1.shape[0]
        net["state_dict"][key] = torch.cat((weight1, torch.randn(delta, weight1.shape[1]).to(device)))

        deltas[key] = delta

    model.load_state_dict(net["state_dict"])

    torch.save({"deltas": deltas, "state_dict": model.state_dict()}, output_path)


if __name__ == "__main__":
    arg_parser = ArgumentParser()
    arg_parser.add_argument("checkpoint", type=str)
    arg_parser.add_argument("config", type=str)
    arg_parser.add_argument("output", type=str)
    args = arg_parser.parse_args()

    expand_weights(args.checkpoint, args.config, args.output)



from os.path import join
from typing import Tuple

import hydra
import torch
from omegaconf import DictConfig
from pytorch_lightning import seed_everything, Trainer, LightningModule, LightningDataModule
from pytorch_lightning.callbacks import ModelCheckpoint, EarlyStopping, LearningRateMonitor
from pytorch_lightning.loggers import WandbLogger

from code2seq.dataset import PathContextDataModule, TypedPathContextDataModule
from code2seq.model import Code2Seq, Code2Class, TypedCode2Seq
from code2seq.utils.callback import UploadCheckpointCallback, PrintEpochResultCallback
from code2seq.utils.common import print_config, filter_warnings
from code2seq.utils.vocabulary import Vocabulary


def get_code2seq(config: DictConfig, vocabulary: Vocabulary) -> Tuple[LightningModule, LightningDataModule]:
    model = Code2Seq(config, vocabulary)
    data_module = PathContextDataModule(config, vocabulary)
    return model, data_module


def get_code2class(config: DictConfig, vocabulary: Vocabulary) -> Tuple[LightningModule, LightningDataModule]:
    model = Code2Class(config, vocabulary)
    data_module = PathContextDataModule(config, vocabulary)
    return model, data_module


def get_typed_code2seq(config: DictConfig, vocabulary: Vocabulary) -> Tuple[LightningModule, LightningDataModule]:
    model = TypedCode2Seq(config, vocabulary)
    data_module = TypedPathContextDataModule(config, vocabulary)
    return model, data_module


def add_mask(weight, size):
    dev = torch.device("cuda")
    mask = torch.zeros_like(weight).to(dev)
    mask[-size:] = 1.

    def tmp(grad):
        return grad * mask

    weight.register_hook(tmp)


@hydra.main(config_path="configs", config_name="code2seq")
def train(config: DictConfig):
    filter_warnings()
    print_config(config)
    seed_everything(config.seed)

    known_models = {"code2seq": get_code2seq, "code2class": get_code2class, "typed-code2seq": get_typed_code2seq}
    if config.name not in known_models:
        print(f"Unknown model: {config.name}, try on of {known_models.keys()}")

    vocabulary = Vocabulary.load_vocabulary(join(config.data_folder, config.dataset.name, config.vocabulary_name))
    model, data_module = known_models[config.name](config, vocabulary)

    expanded_states = torch.load(config.expanded_path)

    model.load_state_dict(expanded_states["state_dict"])

    # define logger
    wandb_logger = WandbLogger(
        project=f"{config.name}-{config.dataset.name}", log_model=True, offline=config.log_offline
    )
    wandb_logger.watch(model)
    # define model checkpoint callback
    checkpoint_callback = ModelCheckpoint(
        dirpath=wandb_logger.experiment.dir,
        filename="{epoch:02d}-{val_loss:.4f}",
        period=config.save_every_epoch,
        save_top_k=-1,
    )
    upload_checkpoint_callback = UploadCheckpointCallback(wandb_logger.experiment.dir)
    # define early stopping callback
    early_stopping_callback = EarlyStopping(
        patience=config.hyper_parameters.patience, monitor="val_loss", verbose=True, mode="min"
    )
    # define callback for printing intermediate result
    print_epoch_result_callback = PrintEpochResultCallback("train", "val")
    # use gpu if it exists
    gpu = 1 if torch.cuda.is_available() else None
    # define learning rate logger
    lr_logger = LearningRateMonitor("step")
    trainer = Trainer(
        max_epochs=config.hyper_parameters.n_epochs,
        gradient_clip_val=config.hyper_parameters.clip_norm,
        deterministic=True,
        check_val_every_n_epoch=config.val_every_epoch,
        log_every_n_steps=config.log_every_epoch,
        logger=wandb_logger,
        gpus=gpu,
        progress_bar_refresh_rate=config.progress_bar_refresh_rate,
        callbacks=[
            lr_logger,
            early_stopping_callback,
            checkpoint_callback,
            upload_checkpoint_callback,
            print_epoch_result_callback,
        ],
        resume_from_checkpoint=config.resume_from_checkpoint,
    )

    for param in model.parameters():
        param.requires_grad = False

    for param in model.encoder.node_embedding.parameters():
        param.requires_grad = True

    add_mask(model.encoder.node_embedding.weight, expanded_states["deltas"]["encoder.node_embedding.weight"])

    trainer.fit(model=model, datamodule=data_module)
    trainer.test()


if __name__ == "__main__":
    train()



from argparse import ArgumentParser
from io import FileIO
from os import path, remove
from typing import Dict

import numpy
from tqdm import tqdm

from random import shuffle, seed

from pathlib import Path

import multiprocessing as mp


def _get_id2value_from_csv(path_: str) -> Dict[str, str]:
    return dict(numpy.genfromtxt(path_, delimiter=",", dtype=(str, str))[1:])


def preprocess_csv(data_folder: str, holdout_name: str, is_shuffled: bool, c2s_output: FileIO):
    """
    Preprocessing for files tokens.csv, paths.csv, node_types.csv
    """
    id_to_token_data_path = path.join(data_folder, f"tokens.csv")
    id_to_type_data_path = path.join(data_folder, f"node_types.csv")
    id_to_paths_data_path = path.join(data_folder, f"paths.csv")
    path_contexts_path = path.join(data_folder, f"path_contexts.csv")

    id_to_paths_stored = _get_id2value_from_csv(id_to_paths_data_path)
    id_to_paths = {index: [n for n in nodes.split()] for index, nodes in id_to_paths_stored.items()}

    id_to_node_types = _get_id2value_from_csv(id_to_type_data_path)
    id_to_node_types = {index: node_type.rsplit(" ", maxsplit=1)[0] for index, node_type in id_to_node_types.items()}

    id_to_tokens = _get_id2value_from_csv(id_to_token_data_path)

    with open(path_contexts_path, "r") as path_contexts_file:
        output_lines = []
        for line in path_contexts_file:
            label, *path_contexts = line.split()
            parsed_line = [label]
            for path_context in path_contexts:
                from_token_id, path_types_id, to_token_id = path_context.split(",")
                from_token, to_token = id_to_tokens[from_token_id], id_to_tokens[to_token_id]
                nodes = [id_to_node_types[p_] for p_ in id_to_paths[path_types_id]]
                parsed_line.append(",".join([from_token, "|".join(nodes), to_token]))
            output_lines.append(" ".join(parsed_line + ["\n"]))
        if is_shuffled:
            shuffle(output_lines)
        c2s_output.write("".join(output_lines))


if __name__ == "__main__":
    arg_parser = ArgumentParser()
    arg_parser.add_argument("data", type=str)
    arg_parser.add_argument("output", type=str)
    arg_parser.add_argument("holdout", type=str)
    arg_parser.add_argument("--shuffle", action="store_true")
    args = arg_parser.parse_args()

    seed(7)

    output_c2s_path = path.join(args.output, f"data.{args.holdout}.c2s")

    if path.exists(output_c2s_path):
        remove(output_c2s_path)

    paths = list(Path(args.data).glob("*/"))

    with open(output_c2s_path, "a+") as c2s_output, mp.Pool(4) as p:
        for project_path in tqdm(paths):
            try:
                preprocess_csv(path.join(str(project_path), "kt"), args.holdout, args.shuffle, c2s_output)

            except ValueError:
                print(project_path)



from typing import List
from dataclasses import dataclass

from pycode2seq.inference.common.node import Node


@dataclass
class MethodNode:
    root: Node
    return_type_node: Node
    name_node: Node

    @property
    def name(self):
        return self.name_node.get_token()

    @property
    def return_type(self):
        return self.return_type_node.get_token()


@dataclass
class ElementNode:
    root: Node
    name_node: Node

    @property
    def name(self):
        return self.name_node.get_token()


@dataclass
class ParameterNode:
    root: Node
    return_type_node: Node
    name_node: Node

    @property
    def name(self):
        return self.name_node.get_token()

    @property
    def return_type(self):
        return self.return_type_node.get_token()


@dataclass
class MethodInfo:
    method: MethodNode
    enclosing_element: ElementNode
    method_parameters: List[ParameterNode]

    @property
    def name(self):
        return self.method.name

    @property
    def return_type(self):
        return self.method.return_type

    @property
    def enclosing_element_name(self):
        return self.enclosing_element.name



import regex as re

from typing import List

from pycode2seq.inference.common.node import Node


TECHNICAL_TOKEN_KEY = "technical_token"
DEFAULT_TOKEN = "EMPTY_TOKEN"


def do_traverse_pre_order(node: Node, result: List[Node]):
    result.append(node)
    for child in node.children:
        do_traverse_pre_order(child, result)


def pre_order(root: Node) -> List[Node]:
    result = []
    do_traverse_pre_order(root, result)
    return result


def set_node_technical_token(node: Node, token: str):
    node.metadata[TECHNICAL_TOKEN_KEY] = token


def normalize_token(token: str, default: str) -> str:
    clean_token = re.sub("\\P{Print}", "", re.sub("[\"',]", "", re.sub("//s+", "", re.sub("\\\\n", "", token.lower()))))

    stripped = re.sub("[^A-Za-z]", "", clean_token)

    if not stripped:
        careful_stripped = clean_token.replace(" ", "_")
        if not careful_stripped:
            return default
        return careful_stripped

    return stripped


def split_to_subtokens(token: str) -> List[str]:
    splitted = re.split("(?<=[a-z])(?=[A-Z])|_|[0-9]|(?<=[A-Z])(?=[A-Z][a-z])|\\s+", token.strip())
    return [normalize_token(token, "") for token in splitted if token]



from abc import ABC, abstractmethod
from typing import List, Optional, Callable

from antlr4 import FileStream, CommonTokenStream, Lexer, InputStream, Parser as AntlrParser, \
    TokenStream, ParserRuleContext, TerminalNode

from pycode2seq.inference.common.node import Node


class Parser(ABC):
    @abstractmethod
    def _lexer(self, input_stream: InputStream) -> Lexer:
        raise NotImplementedError

    @abstractmethod
    def _parser(self, stream: TokenStream) -> AntlrParser:
        raise NotImplementedError

    @abstractmethod
    def _get_tree(self, parser: AntlrParser) -> ParserRuleContext:
        raise NotImplementedError

    def parse(self, file_path: str) -> Node:
        input_stream = FileStream(file_path)
        lexer = self._lexer(input_stream)
        stream = CommonTokenStream(lexer)
        parser = self._parser(stream)
        tree = self._get_tree(parser)

        return Parser._compress_tree(Parser._convert_rule_context(tree, parser.ruleNames, None, lexer))

    @staticmethod
    def _convert_rule_context(cntx: ParserRuleContext, rule_names: List[str], parent: Optional[Node],
                              lexer: Lexer) -> Node:
        type_label = rule_names[cntx.getRuleIndex()]
        current_node = Node(type_label, parent, None)
        children = []
        for child in cntx.getChildren():
            if isinstance(child, TerminalNode):
                children.append(Parser._convert_terminal(child, current_node, lexer))
            else:
                children.append(Parser._convert_rule_context(child, rule_names, current_node, lexer))

        current_node.set_children(children)

        return current_node

    @staticmethod
    def _convert_terminal(terminal_node: TerminalNode, parent: Node, lexer: Lexer):
        return Node(lexer.symbolicNames[terminal_node.getSymbol().type], parent, terminal_node.getSymbol().text)

    @staticmethod
    def _compress_tree(root: Node) -> Node:
        if len(root.children) == 1:
            child = Parser._compress_tree(root.children[0])
            compressed_node = Node(
                root.type_label + "|" + child.type_label,
                root.parent,
                child.get_token()
            )
            compressed_node.set_children(child.children)
            return compressed_node

        root.set_children([Parser._compress_tree(child) for child in root.children])
        return root

    @staticmethod
    def _decompress_type_label(type_label: str) -> List[str]:
        return type_label.split("|")

    @staticmethod
    def _shorten_nodes(root: Node) -> Node:
        parts = Parser._decompress_type_label(root.type_label)
        label = parts[0]

        if len(parts) > 1:
            label += "/" + parts[-1]

        children = [Parser._shorten_nodes(child) for child in root.children]

        new_node = Node(label, root.parent, root.token)

        new_node.set_children(children)

        return new_node



from typing import List


class Node:

    def __init__(self, type_label: str, parent: "Node", token: str) -> None:
        self.type_label = type_label
        self.parent = parent
        self.token = token
        self.metadata = {}
        self.children: List["Node"] = []

    def get_token(self) -> str:
        if self.token is None:
            return "None"
        return self.token

    def get_normalized_token(self) -> str:
        from pycode2seq.inference.common.utils import normalize_token
        if self.token is None:
            return ""
        return normalize_token(self.token, "")

    def pretty_print(self, indent=0, indent_symbol="--") -> None:
        print(indent_symbol * indent, end="")
        print(self.type_label, end="")

        if self.get_token():
            print(f" : {self.get_token()}")
        else:
            print()

        for child in self.children:
            child.pretty_print(indent + 1, indent_symbol)

    def set_children(self, new_children) -> None:
        self.children = new_children
        for child in self.children:
            child.parent = self

    def get_children_of_type(self, type_label: str) -> List["Node"]:
        from pycode2seq.inference.parsing.utils import decompress_type_label
        return [child for child in self.children if
                next(iter(decompress_type_label(child.type_label)), None) == type_label]

    def get_child_of_type(self, type_label: str) -> "Node":
        return next(iter(self.get_children_of_type(type_label)), None)

    def is_leaf(self):
        return len(self.children) == 0



from typing import Callable

from antlr4 import Lexer, InputStream, Parser as AntlrParser, TokenStream, ParserRuleContext

from pycode2seq.inference.parsing.parser import Parser
from pycode2seq.inference.antlr.Java8Lexer import Java8Lexer
from pycode2seq.inference.antlr.Java8Parser import Java8Parser
from pycode2seq.inference.antlr.KotlinLexer import KotlinLexer
from pycode2seq.inference.antlr.KotlinParser import KotlinParser as AntlrKotlinParser


class KotlinParser(Parser):
    def _lexer(self, input_stream: InputStream) -> Lexer:
        return KotlinLexer(input_stream)

    def _parser(self, stream: TokenStream) -> AntlrParser:
        return AntlrKotlinParser(stream)

    def _get_tree(self, parser: AntlrParser) -> ParserRuleContext:
        if not isinstance(parser, AntlrKotlinParser):
            raise ValueError("Wrong parser")
        return parser.kotlinFile()


class JavaParser(Parser):
    def _lexer(self, input_stream: InputStream) -> Lexer:
        return Java8Lexer(input_stream)

    def _parser(self, stream: TokenStream) -> AntlrParser:
        return Java8Parser(stream)

    def _get_tree(self, parser: AntlrParser) -> ParserRuleContext:
        if not isinstance(parser, Java8Parser):
            raise ValueError("Wrong parser")
        return parser.compilationUnit()



from typing import List

from pycode2seq.inference.common.node import Node
from pycode2seq.inference.methods.model import MethodInfo
from pycode2seq.inference.methods.splitter import Splitter
from pycode2seq.inference.methods.splitting_java import JavaSplitter
from pycode2seq.inference.methods.splitting_kotlin import KotlinSplitter
from pycode2seq.inference.parsing.languages import JavaParser, KotlinParser
from pycode2seq.inference.parsing.parser import Parser


class Language:
    def __init__(self, parser: Parser, splitter: Splitter):
        self.parser = parser
        self.splitter = splitter

    def parse(self, file_path: str) -> Node:
        return self.parser.parse(file_path)

    def split_on_methods(self, root: Node) -> List[MethodInfo]:
        return self.splitter.split_on_methods(root)

    @staticmethod
    def by_name(name: str) -> 'Language':
        if name == "java":
            return java
        elif name == "kt" or name == "kotlin":
            return kotlin
        else:
            raise ValueError(f"Unknown language {name}")


java = Language(JavaParser(), JavaSplitter())
kotlin = Language(KotlinParser(), KotlinSplitter())



from setuptools import setup, find_packages

VERSION = "0.0.2"

with open("README.md") as readme_file:
    readme = readme_file.read()

with open("requirements.txt") as requirements_file:
    install_requires = [line.strip() for line in requirements_file]

setup_args = dict(
    name="pycode2seq",
    version=VERSION,
    description="Inference and training for multiple languages of code2seq",
    long_description_content_type="text/markdown",
    long_description=readme,
    install_requires=install_requires,
    license="MIT",
    packages=find_packages(),
    author="Dmitrii Kharlapenko",
    author_email="dimkakha@gmail.com",
    keywords=["code2seq", "pytorch", "pytorch-lightning", "ml4code", "ml4se"],
    url="https://github.com/kisate/pycode2seq",
    download_url="https://pypi.org/project/pycode2seq/",
)

if __name__ == "__main__":
    setup(**setup_args)



import os
import subprocess
import functools

from pathlib import Path
from tqdm import tqdm
from argparse import ArgumentParser

import multiprocessing as mp

holdouts = ["train", "test", "val"]


def call_astminer(path, output_path, cli_path):
    subprocess.call(
        f"./{cli_path} code2vec --lang kt --project {path} --output {Path(output_path, path.name)} --split-tokens --granularity method --hide-method-name",
        shell=True)


def process_holdout(data_path, output_path, cli_path):
    if not Path(output_path).exists():
        os.mkdir(output_path)

    paths = list(Path(data_path).glob("*/"))

    _foo = functools.partial(call_astminer, output_path=output_path, cli_path=cli_path)
    with mp.Pool(4) as p:
        list(tqdm(p.imap(_foo, paths), total=len(paths)))


if __name__ == "__main__":
    arg_parser = ArgumentParser()
    arg_parser.add_argument("data", type=str)
    arg_parser.add_argument("output", type=str)
    arg_parser.add_argument("cli_path", type=str, help="path to astminer cli.sh")

    args = arg_parser.parse_args()

    for holdout in holdouts:
        process_holdout(os.path.join(args.data, holdout), os.path.join(args.output, holdout), args.cli_path)



from pycode2seq.inference.methods.model import MethodInfo
from pycode2seq.inference.common.utils import pre_order, set_node_technical_token, split_to_subtokens


def extract_label(method_info: MethodInfo, hide_method_names: bool) -> str:
    if method_info.method.name_node is None:
        return None
    if method_info.name is None:
        return None

    method_name_node = method_info.method.name_node
    method_root = method_info.method.root
    method_name = method_info.name

    if hide_method_names:
        for node in pre_order(method_root):
            if node.token == method_name:
                set_node_technical_token(node, "SELF")

        set_node_technical_token(method_name_node, "METHOD_NAME")

    return "|".join(split_to_subtokens(method_name))
